#include <hwServer.h>
#include <stdlib.h>
#include <iostream>
#include <hwApp.h>
#include <sstream>
#include <time.h>
#include <pthread.h>
#include <md5.h>
#include <hwWidget.h>
#include <wsServer.h>
#include <wsRequest.h>

hwServer* hwServer::myInstance = NULL;

hwServer::hwServer()
        : myMD5(new MD5())
{
    pthread_mutex_init(&myMutex1, NULL);
    pthread_mutex_init(&myMutex2, NULL);
}

hwServer* hwServer::getInstance()
{
    if (!myInstance)
    {
        hwWidget::init();
        myInstance = new hwServer();
    }

    return myInstance;
}

void hwServer::run(unsigned int port)
{
    wsServer* server = new wsServer(port);
    
    while (wsRequest* req = server->getRequest())
    {
        pthread_t th;
        pthread_create(&th, NULL, hwServer::process, req);
    }
}

void* hwServer::process(void* data)
{
    pthread_mutex_lock(&myInstance->myMutex1);

    wsRequest* request = (wsRequest*)data;
    
    string hwid = request->getParam("hwid");
    hwApp* app;

    pthread_t thread = pthread_self();

    if (!myInstance->myAppMap[hwid])
    {
        time_t secs;
        time(&secs);
        srand(secs);
        stringstream s;
        s << rand();
        string key = myInstance->createKey(s.str());
        app = myInstance->myAppFactory->createInstance();

#ifdef WIN32
        myInstance->myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)] = app;
#else
        myInstance->myThreadMap[thread] = app;
#endif

        app->setId(key);
        app->init();
        cout << "Create App Instance " << key << endl;
        myInstance->myAppMap[key] = app;
        request->setCookie("hwid", key);
    }
    else
    {
        app = myInstance->myAppMap[hwid];
        
#ifdef WIN32
        myInstance->myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)] = app;
#else
        myInstance->myThreadMap[thread] = app;
#endif

    }

    pthread_mutex_unlock(&myInstance->myMutex1);

    app->process(request);
}

hwApp* hwServer::getThreadApp(pthread_t thread)
{
#ifdef WIN32
    return myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)];
#else
    return myThreadMap[thread];
#endif
}

void hwServer::threadDone(pthread_t thread)
{
    pthread_mutex_lock(&myInstance->myMutex2);

#ifdef WIN32
    myThreadMap.erase((unsigned long)pthread_getw32threadhandle_np(thread));
#else
    myThreadMap.erase(thread);
#endif

    pthread_mutex_unlock(&myInstance->myMutex2);
}

string hwServer::createKey(const string& s)
{
    MD5_CTX ctx;

    myMD5->MD5Init(&ctx);
    myMD5->MD5Update(&ctx, (unsigned char*)s.c_str(), s.length());

    unsigned char buff[16] = "";
    myMD5->MD5Final((unsigned char*)buff, &ctx);

    char key[33];

    int p = 0;

    for(int i = 0; i < 16; i++)
    {
        sprintf(&key[p], "%02x", buff[i]);
        p += 2;
    }

    key[32] = '\0';

    return std::string(key);
}

