#include <wsRequest.h>
#include <stdio.h>
#include <iostream>
#include <wsStringTokenizer.h>
#include <stdlib.h>
#include <sstream>
#include <string.h>

#ifdef WIN32
#include <io.h>
#include <winsock2.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

wsRequest::wsRequest(int socket)
        : mySocket(socket)
{
    char data[1024 + 1];
    int bytes = 0;
    string buffer;

    /* Read all data off socket */

    do
    {
        memset(data, 0, sizeof(data));
        bytes = recv(mySocket, data, sizeof(data), 0);
        buffer += data;
    }
    while (bytes >= 1024);

    wsStringTokenizer t1(buffer, "\r\n");

    if (t1.countTokens())
    {
        /* Parse init string */

        string init = t1.nextToken();
        wsStringTokenizer t2(init, " ");
        string method = t2.nextToken();
        string path = t2.nextToken();
        string version = t2.nextToken();

        /* Read the header values */

        bool in_header = true;

        while (in_header)
        {
            string token = t1.nextToken();

            if (token.size() == 0)
            {
                in_header = false;
            }
            else
            {
                size_t p = token.find(":");

                if (p != string::npos)
                {
                    string key = strip(toLower(token.substr(0, p)));
                    string val = strip(token.substr(p + 1));
                    myHeaderVals[key] = val;
                }
            }
        }

        /* Read any content */

        if (myHeaderVals.find("content-length") != myHeaderVals.end())
        {
            int length = atoi(myHeaderVals["content-length"].c_str());
            myContent = buffer.substr(buffer.find("\r\n\r\n")).substr(4, length);
        }

        /* Process cookie */

        if (myHeaderVals.find("cookie") != myHeaderVals.end())
        {
            string cookie = myHeaderVals["cookie"];
            string key = cookie.substr(0, cookie.find("="));
            string val = cookie.substr(cookie.find("=") + 1);
            myParamVals[key] = val;
        }

        /* Process the request */

        if (method == "GET")
        {
            size_t p = path.find("?");

            if (p != string::npos)
            {
                string query = path.substr(p + 1);
                parseQuery(query);
            }
        }
        else if (method == "POST")
        {
            parseQuery(myContent);
        }
    }
}

void wsRequest::parseQuery(const string& query)
{
    wsStringTokenizer t1(query, "&");

    while (t1.hasMoreTokens())
    {
        wsStringTokenizer t2(t1.nextToken(), "=");
        string param = t2.nextToken();
        string val = t2.nextToken();
        myParamVals[param] = decode(val);
        myParamList.push_back(param);
        
        cout << "param " << param << " = " << val << endl;
    }
}

wsRequest::~wsRequest()
{
    close(mySocket);
}

void wsRequest::output(const string& data)
{
    stringstream stream;
    stream << "HTTP/1.0 200 OK\r\n";
    stream << "Content-Type: text/html\r\n";
    stream << "Content-Length: " << data.size() << "\r\n";

    map<string,string>::iterator iter = myResponseVals.begin();

    while (iter != myResponseVals.end())
    {
        stream << (*iter).first << ": " << (*iter).second << "\r\n";
        iter++;
    }

    stream << "\r\n";
    stream << data;
    string output = stream.str();
    send(mySocket, output.c_str(), output.size(), 0);
}

string wsRequest::toLower(const string& s)
{
    string result;

    for (unsigned int i = 0; i < s.size(); i++)
    {
        result += tolower(s[i]);
    }

    return result;
}

string wsRequest::strip(const string& s)
{
    int i1 = s.find_first_not_of(" ");
    int i2 = s.find_last_not_of(" ");
    return s.substr(i1, i2 - i1 + 1);
}

string wsRequest::decode(const string& s)
{
    string result;
    
    if (s.find("%") != string::npos)
    {
        wsStringTokenizer t1(s, "%");
        
        result = t1.nextToken();
        
        while (t1.hasMoreTokens())
        {
            string token = t1.nextToken();
            char code = strtoul(token.substr(0, 2).c_str(), NULL, 16);
            result += code;
            result += token.substr(2);
        }
    }
    else
    {
        result = s;
    }
    
    return result;
}

